@@ -13,6 +13,8 @@
 
 #include "document.h"
 
+#include "synctex/synctex_parser.h"
+
 // qt/kde/system includes
 #include <QtCore/QHash>
 #include <QtCore/QLinkedList>
@@ -96,9 +98,9 @@ class DocumentPrivate
             m_scripter( 0 ),
             m_archiveData( 0 ),
             m_fontsCached( false ),
-            m_documentInfo( 0 ),
             m_annotationEditingEnabled ( true ),
-            m_annotationBeingMoved( false )
+            m_annotationBeingMoved( false ),
+            m_synctex_scanner( 0 )
         {
             calculateMaxTextPages();
         }
@@ -183,6 +185,9 @@ class DocumentPrivate
          */
         bool isNormalizedRectangleFullyVisible( const Okular::NormalizedRect & rectOfInterest, int rectPage );
 
+        // For sync files
+        void loadSyncFile( const QString & filePath );
+
         // member variables
         Document *m_parent;
         QPointer<QWidget> m_widget;
@@ -258,7 +263,8 @@ class DocumentPrivate
 
         QPointer< FontExtractionThread > m_fontThread;
         bool m_fontsCached;
-        DocumentInfo *m_documentInfo;
+        QSet<DocumentInfo::Key> m_documentInfoAskedKeys;
+        DocumentInfo m_documentInfo;
         FontInfo::List m_fontsCache;
 
         QSet< View * > m_views;
@@ -270,6 +276,15 @@ class DocumentPrivate
 
         QUndoStack *m_undoStack;
         QDomNode m_prevPropsOfAnnotBeingModified;
+
+        synctex_scanner_t m_synctex_scanner;
+};
+
+class DocumentInfoPrivate
+{
+    public:
+        QMap<QString, QString> values; // key -> value
+        QMap<QString, QString> titles; // key -> title For the custom keys
 };
 
 }
