@@ -64,6 +64,8 @@
 #include <kdeprintdialog.h>
 #include <kprintpreview.h>
 #include <kbookmarkmenu.h>
+#include <kpassworddialog.h>
+#include <kwallet.h>
 
 // local includes
 #include "aboutdata.h"
@@ -424,19 +426,34 @@ m_cliPresentation(false), m_cliPrint(false), m_embedMode(detectEmbedMode(parentW
     rightLayout->setSpacing( 0 );
     //	KToolBar * rtb = new KToolBar( rightContainer, "mainToolBarSS" );
     //	rightLayout->addWidget( rtb );
-    m_topMessage = new PageViewTopMessage( rightContainer );
-    m_topMessage->setup( i18n( "This document has embedded files. <a href=\"okular:/embeddedfiles\">Click here to see them</a> or go to File -> Embedded Files." ), KIcon( "mail-attachment" ) );
-    connect( m_topMessage, SIGNAL(action()), this, SLOT(slotShowEmbeddedFiles()) );
+    m_topMessage = new KMessageWidget( rightContainer );
+    m_topMessage->setVisible( false );
+    m_topMessage->setWordWrap( true );
+    m_topMessage->setMessageType( KMessageWidget::Information );
+    m_topMessage->setText( i18n( "This document has embedded files. <a href=\"okular:/embeddedfiles\">Click here to see them</a> or go to File -> Embedded Files." ) );
+    m_topMessage->setIcon( KIcon( "mail-attachment" ) );
+    connect( m_topMessage, SIGNAL(linkActivated(QString)), this, SLOT(slotShowEmbeddedFiles()) );
     rightLayout->addWidget( m_topMessage );
-    m_formsMessage = new PageViewTopMessage( rightContainer );
+    m_formsMessage = new KMessageWidget( rightContainer );
+    m_formsMessage->setVisible( false );
+    m_formsMessage->setWordWrap( true );
+    m_formsMessage->setMessageType( KMessageWidget::Information );
     rightLayout->addWidget( m_formsMessage );
+    m_infoMessage = new KMessageWidget( rightContainer );
+    m_infoMessage->setVisible( false );
+    m_infoMessage->setWordWrap( true );
+    m_infoMessage->setMessageType( KMessageWidget::Information );
+    rightLayout->addWidget( m_infoMessage );
+    m_infoTimer = new QTimer();
+    m_infoTimer->setSingleShot( true );
+    connect( m_infoTimer, SIGNAL(timeout()), m_infoMessage, SLOT(animatedHide()) );
     m_pageView = new PageView( rightContainer, m_document );
     QMetaObject::invokeMethod( m_pageView, "setFocus", Qt::QueuedConnection );      //usability setting
 //    m_splitter->setFocusProxy(m_pageView);
     connect( m_pageView, SIGNAL(rightClick(const Okular::Page*,QPoint)), this, SLOT(slotShowMenu(const Okular::Page*,QPoint)) );
-    connect( m_document, SIGNAL(error(QString,int)), m_pageView, SLOT(errorMessage(QString,int)) );
-    connect( m_document, SIGNAL(warning(QString,int)), m_pageView, SLOT(warningMessage(QString,int)) );
-    connect( m_document, SIGNAL(notice(QString,int)), m_pageView, SLOT(noticeMessage(QString,int)) );
+    connect( m_document, SIGNAL(error(QString,int)), this, SLOT(errorMessage(QString,int)) );
+    connect( m_document, SIGNAL(warning(QString,int)), this, SLOT(warningMessage(QString,int)) );
+    connect( m_document, SIGNAL(notice(QString,int)), this, SLOT(noticeMessage(QString,int)) );
     connect( m_document, SIGNAL(sourceReferenceActivated(const QString&,int,int,bool*)), this, SLOT(slotHandleActivatedSourceReference(const QString&,int,int,bool*)) );
     rightLayout->addWidget( m_pageView );
     m_findBar = new FindBar( m_document, rightContainer );
@@ -456,6 +473,7 @@ m_cliPresentation(false), m_cliPrint(false), m_embedMode(detectEmbedMode(parentW
     m_pageNumberTool = new MiniBar( 0, m_miniBarLogic );
 
     connect( m_findBar, SIGNAL(forwardKeyPressEvent(QKeyEvent*)), m_pageView, SLOT(externalKeyPressEvent(QKeyEvent*)));
+    connect( m_findBar, SIGNAL(onCloseButtonPressed()), m_pageView, SLOT(setFocus()));
     connect( m_miniBar, SIGNAL(forwardKeyPressEvent(QKeyEvent*)), m_pageView, SLOT(externalKeyPressEvent(QKeyEvent*)));
     connect( m_pageView, SIGNAL(escPressed()), m_findBar, SLOT(resetSearch()) );
     connect( m_pageNumberTool, SIGNAL(forwardKeyPressEvent(QKeyEvent*)), m_pageView, SLOT(externalKeyPressEvent(QKeyEvent*)));
@@ -847,6 +865,7 @@ Part::~Part()
     delete m_pageSizeLabel;
     delete m_reviewsWidget;
     delete m_bookmarkList;
+    delete m_infoTimer;
 
     delete m_document;
 
@@ -1015,7 +1034,7 @@ void Part::slotJobFinished(KJob *job)
 {
     if ( job->error() == KIO::ERR_USER_CANCELED )
     {
-        m_pageView->noticeMessage( i18n( "The loading of %1 has been canceled.", realUrl().pathOrUrl() ) );
+        m_pageView->displayMessage( i18n( "The loading of %1 has been canceled.", realUrl().pathOrUrl() ) );
     }
 }
 
@@ -1142,19 +1161,8 @@ QString Part::currentDocument()
 
 QString Part::documentMetaData( const QString &metaData ) const
 {
-    const Okular::DocumentInfo * info = m_document->documentInfo();
-    if ( info )
-    {
-        QDomElement docElement = info->documentElement();
-        for ( QDomNode node = docElement.firstChild(); !node.isNull(); node = node.nextSibling() )
-        {
-            const QDomElement element = node.toElement();
-            if ( metaData.compare( element.tagName(), Qt::CaseInsensitive ) == 0 )
-                return element.attribute( "value" );
-        }
-    }
-
-    return QString();
+    const Okular::DocumentInfo info = m_document->documentInfo();
+    return info.get( metaData );
 }
 
 
@@ -1201,52 +1209,165 @@ static void addFileToWatcher( KDirWatch *watcher, const QString &filePath)
     if ( fi.isSymLink() ) watcher->addFile( fi.readLink() );
 }
 
-bool Part::openFile()
+Document::OpenResult Part::doOpenFile( const KMimeType::Ptr &mimeA, const QString &fileNameToOpenA, bool *isCompressedFile )
 {
-    KMimeType::Ptr mime;
-    QString fileNameToOpen = localFilePath();
-    const bool isstdin = url().isLocalFile() && url().fileName( KUrl::ObeyTrailingSlash ) == QLatin1String( "-" );
-    const QFileInfo fileInfo( fileNameToOpen );
-    if ( !isstdin && !fileInfo.exists() )
-        return false;
-    if ( !arguments().mimeType().isEmpty() )
-    {
-        mime = KMimeType::mimeType( arguments().mimeType() );
-    }
-    if ( !mime )
-    {
-        mime = KMimeType::findByPath( fileNameToOpen );
-    }
-    bool isCompressedFile = false;
+    Document::OpenResult openResult = Document::OpenError;
     bool uncompressOk = true;
+    KMimeType::Ptr mime = mimeA;
+    QString fileNameToOpen = fileNameToOpenA;
     QString compressedMime = compressedMimeFor( mime->name() );
     if ( compressedMime.isEmpty() )
         compressedMime = compressedMimeFor( mime->parentMimeType() );
     if ( !compressedMime.isEmpty() )
     {
-        isCompressedFile = true;
+        *isCompressedFile = true;
         uncompressOk = handleCompressed( fileNameToOpen, localFilePath(), compressedMime );
         mime = KMimeType::findByPath( fileNameToOpen );
     }
-    bool ok = false;
+    else
+    {
+        *isCompressedFile = false;
+    }
+
     isDocumentArchive = false;
     if ( uncompressOk )
     {
         if ( mime->is( "application/vnd.kde.okular-archive" ) )
         {
-            ok = m_document->openDocumentArchive( fileNameToOpen, url() );
+            openResult = m_document->openDocumentArchive( fileNameToOpen,  url() );
             isDocumentArchive = true;
         }
         else
         {
-            ok = m_document->openDocument( fileNameToOpen, url(), mime );
+            openResult = m_document->openDocument( fileNameToOpen,  url(), mime );
+        }
+
+        // if the file didn't open correctly it might be encrypted, so ask for a pass
+        QString walletName, walletFolder, walletKey;
+        m_document->walletDataForFile(fileNameToOpen, &walletName, &walletFolder, &walletKey);
+        bool firstInput = true;
+        bool triedWallet = false;
+        KWallet::Wallet * wallet = 0;
+        bool keep = true;
+        while ( openResult == Document::OpenNeedsPassword )
+        {
+            QString password;
+
+            // 1.A. try to retrieve the first password from the kde wallet system
+            if ( !triedWallet && !walletKey.isNull() )
+            {
+                const WId parentwid = widget()->effectiveWinId();
+                wallet = KWallet::Wallet::openWallet( walletName, parentwid );
+                if ( wallet )
+                {
+                    // use the KPdf folder (and create if missing)
+                    if ( !wallet->hasFolder( walletFolder ) )
+                        wallet->createFolder( walletFolder );
+                    wallet->setFolder( walletFolder );
+
+                    // look for the pass in that folder
+                    QString retrievedPass;
+                    if ( !wallet->readPassword( walletKey, retrievedPass ) )
+                        password = retrievedPass;
+                }
+                triedWallet = true;
+            }
+
+            // 1.B. if not retrieved, ask the password using the kde password dialog
+            if ( password.isNull() )
+            {
+                QString prompt;
+                if ( firstInput )
+                    prompt = i18n( "Please enter the password to read the document:" );
+                else
+                    prompt = i18n( "Incorrect password. Try again:" );
+                firstInput = false;
+
+                // if the user presses cancel, abort opening
+                KPasswordDialog dlg( widget(), wallet ? KPasswordDialog::ShowKeepPassword : KPasswordDialog::KPasswordDialogFlags() );
+                dlg.setCaption( i18n( "Document Password" ) );
+                dlg.setPrompt( prompt );
+                if( !dlg.exec() )
+                    break;
+                password = dlg.password();
+                if ( wallet )
+                    keep = dlg.keepPassword();
+            }
+
+            // 2. reopen the document using the password
+            if ( mime->is( "application/vnd.kde.okular-archive" ) )
+            {
+                openResult = m_document->openDocumentArchive( fileNameToOpen,  url(), password );
+                isDocumentArchive = true;
+            }
+            else
+            {
+                openResult = m_document->openDocument( fileNameToOpen,  url(), mime, password );
+            }
+
+            // 3. if the password is correct and the user chose to remember it, store it to the wallet
+            if ( openResult == Document::OpenSuccess && wallet && /*safety check*/ wallet->isOpen() && keep )
+            {
+                wallet->writePassword( walletKey, password );
+            }
         }
     }
-    bool canSearch = m_document->supportsSearching();
 
+    return openResult;
+}
+
+bool Part::openFile()
+{
+    QList<KMimeType::Ptr> mimes;
+    QString fileNameToOpen = localFilePath();
+    const bool isstdin = url().isLocalFile() && url().fileName( KUrl::ObeyTrailingSlash ) == QLatin1String( "-" );
+    const QFileInfo fileInfo( fileNameToOpen );
+    if ( !isstdin && !fileInfo.exists() )
+        return false;
+    KMimeType::Ptr pathMime = KMimeType::findByPath( fileNameToOpen );
+    if ( !arguments().mimeType().isEmpty() )
+    {
+        KMimeType::Ptr argMime = KMimeType::mimeType( arguments().mimeType() );
+
+        // Select the "childmost" mimetype, if none of them
+        // inherits the other trust more what pathMime says
+        // but still do a second try if that one fails
+        if ( argMime->is( pathMime->name() ) )
+        {
+            mimes << argMime;
+        }
+        else if ( pathMime->is( argMime->name() ) )
+        {
+            mimes << pathMime;
+        }
+        else
+        {
+            mimes << pathMime << argMime;
+        }
+
+        if (mimes[0]->name() == "text/plain") {
+            KMimeType::Ptr contentMime = KMimeType::findByFileContent( fileNameToOpen );
+            mimes.prepend( contentMime );
+        }
+    }
+    else
+    {
+        mimes << pathMime;
+    }
+
+    KMimeType::Ptr mime;
+    Document::OpenResult openResult = Document::OpenError;
+    bool isCompressedFile = false;
+    while ( !mimes.isEmpty() && openResult == Document::OpenError ) {
+        mime = mimes.takeFirst();
+        openResult = doOpenFile( mime, fileNameToOpen, &isCompressedFile );
+    }
+
+    bool canSearch = m_document->supportsSearching();
     emit mimeTypeChanged( mime );
 
     // update one-time actions
+    const bool ok = openResult == Document::OpenSuccess;
     emit enableCloseAction( ok );
     m_find->setEnabled( ok && canSearch );
     m_findNext->setEnabled( ok && canSearch );
@@ -1263,13 +1384,16 @@ bool Part::openFile()
     // Warn the user that XFA forms are not supported yet (NOTE: poppler generator only)
     if ( ok && m_document->metaData( "HasUnsupportedXfaForm" ).toBool() == true )
     {
-        m_formsMessage->setup( i18n( "This document has XFA forms, which are currently <b>unsupported</b>." ), KIcon( "dialog-warning" ) );
+        m_formsMessage->setText( i18n( "This document has XFA forms, which are currently <b>unsupported</b>." ) );
+        m_formsMessage->setIcon( KIcon( "dialog-warning" ) );
+        m_formsMessage->setMessageType( KMessageWidget::Warning );
         m_formsMessage->setVisible( true );
     }
     // m_pageView->toggleFormsAction() may be null on dummy mode
     else if ( ok && m_pageView->toggleFormsAction() && m_pageView->toggleFormsAction()->isEnabled() )
     {
-        m_formsMessage->setup( i18n( "This document has forms. Click on the button to interact with them, or use View -> Show Forms." ) );
+        m_formsMessage->setText( i18n( "This document has forms. Click on the button to interact with them, or use View -> Show Forms." ) );
+        m_formsMessage->setMessageType( KMessageWidget::Information );
         m_formsMessage->setVisible( true );
     }
     else
@@ -1325,7 +1449,7 @@ bool Part::openFile()
     }
 
     // if the 'OpenTOC' flag is set, open the TOC
-    if ( m_document->metaData( "OpenTOC" ).toBool() && m_sidebar->isItemEnabled( 0 ) && !m_sidebar->isCollapsed() )
+    if ( m_document->metaData( "OpenTOC" ).toBool() && m_sidebar->isItemEnabled( 0 ) && !m_sidebar->isCollapsed() && m_sidebar->currentIndex() != 0 )
     {
         m_sidebar->setCurrentIndex( 0, Sidebar::DoNotUncollapseIfCollapsed );
     }
@@ -1795,7 +1919,7 @@ void Part::enableTOC(bool enable)
     m_sidebar->setItemEnabled(0, enable);
 
     // If present, show the TOC when a document is opened
-    if ( enable )
+    if ( enable && m_sidebar->currentIndex() != 0 )
     {
         m_sidebar->setCurrentIndex( 0, Sidebar::DoNotUncollapseIfCollapsed );
     }
@@ -2445,14 +2569,12 @@ void Part::slotAboutBackend()
 
     if ( aboutData.programIconName().isEmpty() || aboutData.programIconName() == aboutData.appName() )
     {
-        if ( const Okular::DocumentInfo *documentInfo = m_document->documentInfo() )
+        const Okular::DocumentInfo documentInfo = m_document->documentInfo(QSet<DocumentInfo::Key>() << DocumentInfo::MimeType);
+        const QString mimeTypeName = documentInfo.get(DocumentInfo::MimeType);
+        if ( !mimeTypeName.isEmpty() )
         {
-            const QString mimeTypeName = documentInfo->get("mimeType");
-            if ( !mimeTypeName.isEmpty() )
-            {
-                if ( KMimeType::Ptr type = KMimeType::mimeType( mimeTypeName ) )
-                    aboutData.setProgramIconName( type->iconName() );
-            }
+            if ( KMimeType::Ptr type = KMimeType::mimeType( mimeTypeName ) )
+                aboutData.setProgramIconName( type->iconName() );
         }
     }
 
@@ -2657,6 +2779,50 @@ void Part::psTransformEnded(int exit, QProcess::ExitStatus status)
 }
 
 
+void Part::displayInfoMessage( const QString &message, KMessageWidget::MessageType messageType, int duration )
+{
+    if ( !Okular::Settings::showOSD() )
+    {
+        if (messageType == KMessageWidget::Error)
+        {
+            KMessageBox::error( widget(), message );
+        }
+        return;
+    }
+
+    // hide messageWindow if string is empty
+    if ( message.isEmpty() )
+        m_infoMessage->animatedHide();
+
+    // display message (duration is length dependant)
+    if ( duration < 0 )
+    {
+        duration = 500 + 100 * message.length();
+    }
+    m_infoTimer->start( duration );
+    m_infoMessage->setText( message );
+    m_infoMessage->setMessageType( messageType );
+    m_infoMessage->setVisible( true );
+}
+
+
+void Part::errorMessage( const QString &message, int duration )
+{
+    displayInfoMessage( message, KMessageWidget::Error, duration );
+}
+
+void Part::warningMessage( const QString &message, int duration )
+{
+    displayInfoMessage( message, KMessageWidget::Warning, duration );
+}
+
+void Part::noticeMessage( const QString &message, int duration )
+{
+    // less important message -> simpleer display widget in the PageView
+    m_pageView->displayMessage( message, QString(), PageViewMessage::Info, duration );
+}
+
+
 void Part::unsetDummyMode()
 {
     if ( m_embedMode == PrintPreviewMode )
@@ -2678,7 +2844,8 @@ void Part::unsetDummyMode()
     m_pageView->setupActions( actionCollection() );
 
     // attach the actions of the children widgets too
-    m_formsMessage->setActionButton( m_pageView->toggleFormsAction() );
+    m_formsMessage->addAction( m_pageView->toggleFormsAction() );
+    m_formsMessage->setVisible( m_pageView->toggleFormsAction() != 0 );
 
     // ensure history actions are in the correct state
     updateViewActions();
